@@ -12,13 +12,13 @@
 
         ...
 
-        >>> surveys = sg.api.list_surveys()
+        >>> surveys = sg.api.survey.list()
         >>> print surveys
 
 '''
 
 __title__ = 'surveygizmo'
-__version_info__ = ('0', '0', '2')
+__version_info__ = ('0', '0', '3')
 __version__ = '.'.join(__version_info__)
 __author__ = 'Ryan P Kilby'
 __license__ = 'MIT'
