@@ -1,5 +1,6 @@
 
-def list_all(survey_id, *args, **kwargs):
+
+def list(survey_id, *args, **kwargs):
     tail = "survey/%s/surveyresponse" % survey_id
     params = {
 
@@ -36,4 +37,4 @@ def change(survey_id, response_id, *args, **kwargs):
     params = {
         '_method': 'POST',
     }.update(kwargs)
-    return tail, params
\ No newline at end of file
+    return tail, params
